@@ -220,7 +220,6 @@ IsControllerSwitchJoyConPair(GCController *controller)
 static BOOL
 IOS_AddMFIJoystickDevice(SDL_JoystickDeviceItem *device, GCController *controller)
 {
-    Uint16 *guid16 = (Uint16 *)device->guid.data;
     Uint16 vendor = 0;
     Uint16 product = 0;
     Uint8 subtype = 0;
@@ -266,9 +265,6 @@ IOS_AddMFIJoystickDevice(SDL_JoystickDeviceItem *device, GCController *controlle
         BOOL is_ps5 = IsControllerPS5(controller);
         BOOL is_switch_pro = IsControllerSwitchPro(controller);
         BOOL is_switch_joycon_pair = IsControllerSwitchJoyConPair(controller);
-#if TARGET_OS_TV
-        BOOL is_MFi = (!is_xbox && !is_ps4 && !is_ps5 && !is_switch_pro && !is_switch_joycon_pair);
-#endif
         int nbuttons = 0;
         BOOL has_direct_menu;
 
@@ -311,18 +307,19 @@ IOS_AddMFIJoystickDevice(SDL_JoystickDeviceItem *device, GCController *controlle
             device->button_mask |= (1 << SDL_CONTROLLER_BUTTON_GUIDE);
             ++nbuttons;
         }
-        has_direct_menu = [gamepad respondsToSelector:@selector(buttonMenu)] && gamepad.buttonMenu;
-#if TARGET_OS_TV
-        /* On tvOS MFi controller menu button brings you to the home screen */
-        if (is_MFi) {
-            has_direct_menu = FALSE;
-        }
-#endif
         device->button_mask |= (1 << SDL_CONTROLLER_BUTTON_START);
         ++nbuttons;
+
+        has_direct_menu = [gamepad respondsToSelector:@selector(buttonMenu)] && gamepad.buttonMenu;
         if (!has_direct_menu) {
             device->uses_pause_handler = SDL_TRUE;
         }
+#if TARGET_OS_TV
+        /* The single menu button isn't very reliable, at least as of tvOS 16.1 */
+        if ((device->button_mask & (1 << SDL_CONTROLLER_BUTTON_BACK)) == 0) {
+            device->uses_pause_handler = SDL_TRUE;
+        }
+#endif
 
 #ifdef ENABLE_PHYSICAL_INPUT_PROFILE
         if ([controller respondsToSelector:@selector(physicalInputProfile)]) {
@@ -441,9 +438,14 @@ IOS_AddMFIJoystickDevice(SDL_JoystickDeviceItem *device, GCController *controlle
         device->button_mask |= (1 << SDL_CONTROLLER_BUTTON_Y);
         device->button_mask |= (1 << SDL_CONTROLLER_BUTTON_LEFTSHOULDER);
         device->button_mask |= (1 << SDL_CONTROLLER_BUTTON_RIGHTSHOULDER);
+#if TARGET_OS_TV
+        /* The menu button is used by the OS and not available to applications */
+        nbuttons += 6;
+#else
         device->button_mask |= (1 << SDL_CONTROLLER_BUTTON_START);
         nbuttons += 7;
         device->uses_pause_handler = SDL_TRUE;
+#endif
 
         device->naxes = 0; /* no traditional analog inputs */
         device->nhats = 1; /* d-pad */
@@ -472,23 +474,17 @@ IOS_AddMFIJoystickDevice(SDL_JoystickDeviceItem *device, GCController *controlle
     }
 #endif /* TARGET_OS_TV */
 
-    /* We only need 16 bits for each of these; space them out to fill 128. */
-    /* Byteswap so devices get same GUID on little/big endian platforms. */
-    *guid16++ = SDL_SwapLE16(SDL_HARDWARE_BUS_BLUETOOTH);
-    *guid16++ = 0;
-    *guid16++ = SDL_SwapLE16(vendor);
-    *guid16++ = 0;
-    *guid16++ = SDL_SwapLE16(product);
-    *guid16++ = 0;
-
-    *guid16++ = SDL_SwapLE16(device->button_mask);
-
     if (vendor == USB_VENDOR_APPLE) {
         /* Note that this is an MFI controller and what subtype it is */
-        device->guid.data[14] = 'm';
-        device->guid.data[15] = subtype;
+        device->guid = SDL_CreateJoystickGUID(SDL_HARDWARE_BUS_BLUETOOTH, vendor, product, 0, name, 'm', subtype);
     } else {
-        device->guid.data[15] = subtype;
+        device->guid = SDL_CreateJoystickGUID(SDL_HARDWARE_BUS_BLUETOOTH, vendor, product, 0, name, 0, subtype);
+    }
+
+    /* Update the GUID with capability bits */
+    {
+        Uint16 *guid16 = (Uint16 *)device->guid.data;
+        guid16[6] = SDL_SwapLE16(device->button_mask);
     }
 
     /* This will be set when the first button press of the controller is
@@ -639,20 +635,20 @@ SDL_AppleTVRemoteRotationHintChanged(void *udata, const char *name, const char *
 }
 #endif /* TARGET_OS_TV */
 
-#if defined(__MACOSX__)
-static int is_macos11(void)
-{
-    return (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_15);
-}
-#endif
-
 static int
 IOS_JoystickInit(void)
 {
 #if defined(__MACOSX__)
-    if (!is_macos11()) {
+#if _SDL_HAS_BUILTIN(__builtin_available)
+    if (@available(macOS 10.16, *)) {
+        /* Continue with initialization on macOS 11+ */
+    } else {
         return 0;
     }
+#else
+    /* No @available, must be an older macOS version */
+    return 0;
+#endif
 #endif
 
     @autoreleasepool {
@@ -690,7 +686,9 @@ IOS_JoystickInit(void)
                                                queue:nil
                                           usingBlock:^(NSNotification *note) {
                                               GCController *controller = note.object;
+                                              SDL_LockJoysticks();
                                               IOS_AddJoystickDevice(controller, SDL_FALSE);
+                                              SDL_UnlockJoysticks();
                                           }];
 
         disconnectObserver = [center addObserverForName:GCControllerDidDisconnectNotification
@@ -698,14 +696,15 @@ IOS_JoystickInit(void)
                                                   queue:nil
                                              usingBlock:^(NSNotification *note) {
                                                  GCController *controller = note.object;
-                                                 SDL_JoystickDeviceItem *device = deviceList;
-                                                 while (device != NULL) {
+                                                 SDL_JoystickDeviceItem *device;
+                                                 SDL_LockJoysticks();
+                                                 for (device = deviceList; device != NULL; device = device->next) {
                                                      if (device->controller == controller) {
                                                          IOS_RemoveJoystickDevice(device);
                                                          break;
                                                      }
-                                                     device = device->next;
                                                  }
+                                                 SDL_UnlockJoysticks();
                                              }];
 #endif /* SDL_JOYSTICK_MFI */
     }
@@ -957,6 +956,7 @@ IOS_MFIJoystickUpdate(SDL_Joystick *joystick)
 #endif
 
         if (controller.extendedGamepad) {
+            SDL_bool isstack;
             GCExtendedGamepad *gamepad = controller.extendedGamepad;
 
             /* Axis order matches the XInput Windows mappings. */
@@ -970,9 +970,14 @@ IOS_MFIJoystickUpdate(SDL_Joystick *joystick)
             };
 
             /* Button order matches the XInput Windows mappings. */
-            Uint8 buttons[joystick->nbuttons];
+            Uint8 *buttons = SDL_small_alloc(Uint8, joystick->nbuttons, &isstack);
             int button_count = 0;
 
+            if (buttons == NULL) {
+                SDL_OutOfMemory();
+                return;
+            }
+
             /* These buttons are part of the original MFi spec */
             buttons[button_count++] = gamepad.buttonA.isPressed;
             buttons[button_count++] = gamepad.buttonB.isPressed;
@@ -1076,25 +1081,33 @@ IOS_MFIJoystickUpdate(SDL_Joystick *joystick)
                         data[0] = rate.x;
                         data[1] = rate.z;
                         data[2] = -rate.y;
-                        SDL_PrivateJoystickSensor(joystick, SDL_SENSOR_GYRO, data, 3);
+                        SDL_PrivateJoystickSensor(joystick, SDL_SENSOR_GYRO, 0, data, 3);
                     }
                     if (motion.hasGravityAndUserAcceleration) {
                         GCAcceleration accel = motion.acceleration;
                         data[0] = -accel.x * SDL_STANDARD_GRAVITY;
                         data[1] = -accel.y * SDL_STANDARD_GRAVITY;
                         data[2] = -accel.z * SDL_STANDARD_GRAVITY;
-                        SDL_PrivateJoystickSensor(joystick, SDL_SENSOR_ACCEL, data, 3);
+                        SDL_PrivateJoystickSensor(joystick, SDL_SENSOR_ACCEL, 0, data, 3);
                     }
                 }
             }
 #endif /* ENABLE_MFI_SENSORS */
 
+            SDL_small_free(buttons, isstack);
         } else if (controller.gamepad) {
+            SDL_bool isstack;
             GCGamepad *gamepad = controller.gamepad;
 
             /* Button order matches the XInput Windows mappings. */
-            Uint8 buttons[joystick->nbuttons];
+            Uint8 *buttons = SDL_small_alloc(Uint8, joystick->nbuttons, &isstack);
             int button_count = 0;
+
+            if (buttons == NULL) {
+                SDL_OutOfMemory();
+                return;
+            }
+
             buttons[button_count++] = gamepad.buttonA.isPressed;
             buttons[button_count++] = gamepad.buttonB.isPressed;
             buttons[button_count++] = gamepad.buttonX.isPressed;
@@ -1109,6 +1122,8 @@ IOS_MFIJoystickUpdate(SDL_Joystick *joystick)
             for (i = 0; i < button_count; i++) {
                 SDL_PrivateJoystickButton(joystick, i, buttons[i]);
             }
+
+            SDL_small_free(buttons, isstack);
         }
 #if TARGET_OS_TV
         else if (controller.microGamepad) {
@@ -1661,7 +1676,7 @@ IOS_JoystickGetGamepadMapping(int device_index, SDL_GamepadMapping *out)
 #if defined(SDL_JOYSTICK_MFI) && defined(__MACOSX__)
 SDL_bool IOS_SupportedHIDDevice(IOHIDDeviceRef device)
 {
-    if (is_macos11()) {
+    if (@available(macOS 10.16, *)) {
         if ([GCController supportsHIDDevice:device]) {
             return SDL_TRUE;
         }
